/* Dia -- an diagram creation/manipulation program
 * Copyright (C) 1998 Alexander Larsson
 *
 * Jackson diagram -  adapted by Christophe Ponsard
 * This class captures all kind of domains (given, designed, machine)
 * both for generic problems and for problem frames (ie. with domain kinds)
 *
 * based on SADT diagrams copyright (C) 2000, 2001 Cyrille Chepelov
 *
 * Forked from Flowchart toolpc -- objects for drawing flowcharts.
 * Copyright (C) 1999 James Henstridge.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <string.h>
#include <glib.h>

#include "intl.h"
#include "object.h"
#include "element.h"
#include "connectionpoint.h"
#include "diarenderer.h"
#include "attributes.h"
#include "text.h"
#include "widgets.h"
#include "message.h"
#include "connpoint_line.h"
#include "color.h"

#include "logiciel.h"

//#include "menace.h"


#define DEFAULT_WIDTH  1.0
#define DEFAULT_HEIGHT 1.0

 

static ObjectTypeOps logiciel_type_ops =
{
  (CreateFunc) pc_create,
  (LoadFunc)   pc_load,
  (SaveFunc)   object_sissi_save,
  (GetDefaultsFunc)   NULL,
  (ApplyDefaultsFunc) NULL,
};

DiaObjectType logiciel_type =
{
  "SISSI - logiciel",           /* name */
  0,                            /* version */
  (char **) logiciel_xpm, /* this is the default pixmap */
  &logiciel_type_ops      /* ops */
};


static ObjectOps object_sissi_ops = {
  (DestroyFunc)         object_sissi_destroy,
  (DrawFunc)            object_sissi_draw,
  (DistanceFunc)        object_sissi_distance_from,
  (SelectFunc)          object_sissi_select,
  (CopyFunc)            object_sissi_copy_using_properties,
  (MoveFunc)            object_sissi_move,
  (MoveHandleFunc)      object_sissi_move_handle,
  (GetPropertiesFunc)   object_sissi_get_properties,
  (ApplyPropertiesFunc) object_sissi_apply_properties,
  (ObjectMenuFunc)      NULL,
  (DescribePropsFunc)   pc_describe_props,
  (GetPropsFunc)        pc_get_props,
  (SetPropsFunc)        pc_set_props
};

static PropDescription generic_props[] = {
  ELEMENT_COMMON_PROPERTIES,
//      { "confidentialite", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE, N_("Confidentialite :"), N_("confidentialite "), propriete_confidentialite_data},     
    PROP_DESC_END
};

static PropOffset generic_offsets[] = {
  ELEMENT_COMMON_PROPERTIES_OFFSETS,
//    { "confidentialite", PROP_TYPE_ENUM, offsetof(ObjetSISSI,confidentialite)},
  {NULL}
};

static PropDescription *pc_describe_props(ObjetSISSI *pc)
{
//   fprintf(stderr,"espion pc_describe_props debut\t"); 
  if (generic_props[0].quark == 0) {
    prop_desc_list_calculate_quarks(generic_props);
  }
//   fprintf(stderr,"espion pc_describe_props fin\n"); 
  return generic_props;
}

static void pc_set_props(ObjetSISSI *object_sissi, GPtrArray *props)
{
//   object_set_props_from_offsets(&object_sissi->element.object, generic_offsets,props);

//   object_sissi_update_data(object_sissi, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
}

static void pc_get_props(ObjetSISSI *object_sissi, GPtrArray *props)
{
//     fprintf(stderr,"espion pc_get_props debut\t"); 
    object_get_props_from_offsets(&object_sissi->element.object, generic_offsets,props);
//     fprintf(stderr,"espion pc_get_props fin\n"); 
}

/////////////create//////////
static DiaObject *pc_create(Point *startpoint,  void *user_data, Handle **handle1, Handle **handle2)
{
  ObjetSISSI *object_sissi;
  Element *elem;
  DiaObject *obj;
  int i;
//     fprintf(stderr,"espion pc_create debut\t");   
  object_sissi = g_malloc0(sizeof(ObjetSISSI));
  elem = &object_sissi->element;
  obj = &elem->object;

  obj->type = &logiciel_type;
  obj->ops = &object_sissi_ops;

  elem->corner = *startpoint;
  elem->width = DEFAULT_WIDTH;
  elem->height = DEFAULT_HEIGHT;

 element_init(elem, 8, 8);

  for (i=0;i<8;i++) {
    obj->connections[i] = &object_sissi->connections[i];
    object_sissi->connections[i].object = obj;
    object_sissi->connections[i].connected = NULL;
  }
  
   object_sissi->border_color=color_brown;
   object_sissi->fill_colour=color_brown;
///////////////////   Teste si on crer l'objet depuis un appel  la boite  outil ou d'un copier coller ou depuis au quel cas il mettra le chiffre logiciel par dfaut
    if(GPOINTER_TO_INT(user_data)!=0)
	object_sissi->type_element = g_strdup(liste_element[GPOINTER_TO_INT(user_data)]);
    else
    	object_sissi->type_element = g_strdup(liste_element[44]);
    
    if(GPOINTER_TO_INT(user_data)!=0)
	object_sissi->file = g_strdup(liste_image[GPOINTER_TO_INT(user_data)]);
    else
    	object_sissi->file = g_strdup(liste_image[44]);


    object_sissi->image = dia_image_load(dia_get_data_directory());

    if (object_sissi->image) {
      elem->width = (elem->width*(float)dia_image_width(object_sissi->image))/
	(float)dia_image_height(object_sissi->image);
    }

  object_sissi->proprietes_menaces=create_list_proprietes_menace(object_sissi->proprietes_menaces);
  object_sissi->proprietes_autres=create_list_proprietes_autres(object_sissi->proprietes_autres);
  
  object_sissi->entite=g_strdup("");
  object_sissi->entite_type=g_strdup("LOGICIEL");
//   entite_type=g_strdup("LOGICIEL");
//   entite_type=g_strdup("PERSONNE");
//   entite_type=g_strdup("RESEAU");
//   entite_type=g_strdup("PHYSIQUE");
//   entite_type=g_strdup("ORGANISATION");
//   entite_type=g_strdup("SYSTEME");


  object_sissi_update_data(object_sissi, ANCHOR_MIDDLE, ANCHOR_MIDDLE);

  *handle1 = NULL;
  *handle2 = obj->handles[7];
//     fprintf(stderr,"espion pc_create fin\n");     
   return &object_sissi->element.object;
}
 
DiaObject *pc_load(ObjectNode obj_node, int version, const char *filename)
{
  ObjetSISSI *object_sissi;
  Element *elem;
  DiaObject *obj;
  gchar *file_name;
  
  
  object_sissi = g_malloc0(sizeof(ObjetSISSI));
  elem = &object_sissi->element;
  obj = &elem->object;

  obj->type = &logiciel_type;
  obj->ops = &object_sissi_ops;

  object_sissi=object_sissi_load(obj_node, version, filename, object_sissi,elem,obj);
  
  file_name= g_strdup(object_sissi->file); // cette ligne permet dajouter a la fonction dia_get_data_directory() l'url du fichier
  object_sissi->image = dia_image_load(dia_get_data_directory());
  
  object_sissi_update_data(object_sissi, ANCHOR_MIDDLE, ANCHOR_MIDDLE);

  return &object_sissi->element.object;

}

static GList *create_list_proprietes_autres(GList *list_propriete_autre)
{
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Organism in charge","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Function","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Version","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("ITSEC","Software ITSEC Classification", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Description","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Inventory number","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Serial number","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Link system name","Link system name object", "")); 
//     fprintf(stderr,"espion create_list_proprietes_autres fin\n");    
 return list_propriete_autre;
}