/* Dia -- an diagram creation/manipulation program
 * Copyright (C) 1998 Alexander Larsson
 *
 * Jackson diagram -  adapted by Christophe Ponsard
 * This class captures all kind of domains (given, designed, machine)
 * both for generic problems and for problem frames (ie. with domain kinds)
 *
 * based on SADT diagrams copyright (C) 2000, 2001 Cyrille Chepelov
 *
 * Forked from Flowchart toolpersonne -- objects for drawing flowcharts.
 * Copyright (C) 1999 James Henstridge.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <string.h>
#include <glib.h>

#include "intl.h"
#include "object.h"
#include "element.h"
#include "connectionpoint.h"
#include "diarenderer.h"
#include "attributes.h"
#include "text.h"
#include "widgets.h"
#include "message.h"
#include "connpoint_line.h"
#include "color.h"

#include "personne.h"

//#include "menace.h"


#define DEFAULT_WIDTH  1.0
#define DEFAULT_HEIGHT 1.0

 

static ObjectTypeOps personne_type_ops =
{
  (CreateFunc) personne_create,
  (LoadFunc)   personne_load,
  (SaveFunc)   object_sissi_save,
  (GetDefaultsFunc)   NULL,
  (ApplyDefaultsFunc) NULL,
};

DiaObjectType personne_type =
{
  "SISSI - personne",           /* name */
  0,                            /* version */
  (char **) personne_xpm, /* this is the default pixmap */
  &personne_type_ops      /* ops */
};


static ObjectOps object_sissi_ops = {
  (DestroyFunc)         object_sissi_destroy,
  (DrawFunc)            object_sissi_draw,
  (DistanceFunc)        object_sissi_distance_from,
  (SelectFunc)          object_sissi_select,
  (CopyFunc)            object_sissi_copy_using_properties,
  (MoveFunc)            object_sissi_move,
  (MoveHandleFunc)      object_sissi_move_handle,
  (GetPropertiesFunc)   object_sissi_get_properties,
  (ApplyPropertiesFunc) object_sissi_apply_properties,
  (ObjectMenuFunc)      NULL,
  (DescribePropsFunc)   personne_describe_props,
  (GetPropsFunc)        personne_get_props,
  (SetPropsFunc)        personne_set_props
};

static PropDescription generic_props[] = {
  ELEMENT_COMMON_PROPERTIES,
{ "name", PROP_TYPE_STRING, PROP_FLAG_VISIBLE | PROP_FLAG_OPTIONAL, N_("Name"), NULL, NULL }, 
    PROP_DESC_END
};

static PropOffset generic_offsets[] = {
  ELEMENT_COMMON_PROPERTIES_OFFSETS,
// 	{ "name", PROP_TYPE_STRING, offsetof(ObjetSISSI, name) },
  {NULL}
};

static PropDescription *personne_describe_props(ObjetSISSI *personne)
{
  if (generic_props[0].quark == 0) {
    prop_desc_list_calculate_quarks(generic_props);
  }
  return generic_props;
}

static void personne_set_props(ObjetSISSI *object_sissi, GPtrArray *props)
{
//   object_set_props_from_offsets(&object_sissi->element.object, generic_offsets,props);

//   object_sissi_update_data(object_sissi, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
}

static void personne_get_props(ObjetSISSI *object_sissi, GPtrArray *props)
{
  object_get_props_from_offsets(&object_sissi->element.object, generic_offsets,props);
}

/////////////create//////////
static DiaObject *personne_create(Point *startpoint,  void *user_data, Handle **handle1, Handle **handle2)
{
  ObjetSISSI *object_sissi;
  Element *elem;
  DiaObject *obj;
  int i;
  
  object_sissi = g_malloc0(sizeof(ObjetSISSI));
  elem = &object_sissi->element;
  obj = &elem->object;

  obj->type = &personne_type;
  obj->ops = &object_sissi_ops;

  elem->corner = *startpoint;
  elem->width = DEFAULT_WIDTH;
  elem->height = DEFAULT_HEIGHT;

 element_init(elem, 8, 8);

  for (i=0;i<8;i++) {
    obj->connections[i] = &object_sissi->connections[i];
    object_sissi->connections[i].object = obj;
    object_sissi->connections[i].connected = NULL;
  }

    object_sissi->border_color=color_yellow;
   object_sissi->fill_colour=color_yellow;
     
    if(GPOINTER_TO_INT(user_data)!=0)
	object_sissi->type_element = g_strdup(liste_element[GPOINTER_TO_INT(user_data)]);
    else
    	object_sissi->type_element = g_strdup(liste_element[39]);
    
    if(GPOINTER_TO_INT(user_data)!=0)
	object_sissi->file = g_strdup(liste_image[GPOINTER_TO_INT(user_data)]);
    else
    	object_sissi->file = g_strdup(liste_image[39]);
   
   object_sissi->image = dia_image_load(dia_get_data_directory());
  
    if (object_sissi->image) {
      elem->width = (elem->width*(float)dia_image_width(object_sissi->image))/
	(float)dia_image_height(object_sissi->image);
    }

  object_sissi->proprietes_menaces=create_list_proprietes_menace(object_sissi->proprietes_menaces);
  object_sissi->proprietes_autres=create_list_proprietes_autres(object_sissi->proprietes_autres);
  
  object_sissi->entite=g_strdup("");
  object_sissi->entite_type=g_strdup("PERSONNE");
//   entite_type=g_strdup("LOGICIEL");
//   entite_type=g_strdup("PERSONNE");
//   entite_type=g_strdup("RESEAU");
//   entite_type=g_strdup("PHYSIQUE");
//   entite_type=g_strdup("ORGANISATION");
//   entite_type=g_strdup("SYSTEME");


  object_sissi_update_data(object_sissi, ANCHOR_MIDDLE, ANCHOR_MIDDLE);

  *handle1 = NULL;
  *handle2 = obj->handles[7];
//  fprintf (stderr,"espion personne_create debut\t") ;  
   return &object_sissi->element.object;
//  fprintf (stderr,"espion personne_create fin2t\n") ;
}
 
DiaObject *personne_load(ObjectNode obj_node, int version, const char *filename)
{
  ObjetSISSI *object_sissi;
  Element *elem;
  DiaObject *obj;
  gchar *file_name;
  
  
  object_sissi = g_malloc0(sizeof(ObjetSISSI));
  elem = &object_sissi->element;
  obj = &elem->object;

  obj->type = &personne_type;
  obj->ops = &object_sissi_ops;

  object_sissi=object_sissi_load(obj_node, version, filename, object_sissi,elem,obj);
  
  file_name= g_strdup(object_sissi->file); // cette ligne permet dajouter a la fonction dia_get_data_directory() l'url du fichier
  object_sissi->image = dia_image_load(dia_get_data_directory());
  
  object_sissi_update_data(object_sissi, ANCHOR_MIDDLE, ANCHOR_MIDDLE);

  return &object_sissi->element.object;

}

static GList *create_list_proprietes_autres(GList *list_propriete_autre)
{
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("First name","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Organisme d'appartenance","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Function","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Description","", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Adress","Mail adress", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("TEL","Office telephone", ""));
 list_propriete_autre = g_list_append(list_propriete_autre, create_new_propriete_autre("Cellular phone","Constraint cellular", ""));
 
 return list_propriete_autre;
}