/* Dia -- an diagram creation/manipulation program
 * Copyright (C) 1998 Alexander Larsson
 *
 * Jackson diagram -  adapted by Christophe Ponsard
 * This class captures all kind of domains (given, designed, machine)
 * both for generic problems and for problem frames (ie. with domain kinds)
 *
 * based on SADT diagrams copyright (C) 2000, 2001 Cyrille Chepelov
 *
 * Forked from Flowchart toolbox -- objects for drawing flowcharts.
 * Copyright (C) 1999 James Henstridge.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef SISSI_H
#define SISSI_H

#include <stdio.h>
#include <assert.h>
#include <glib.h>

#include "diatypes.h"
#include <gdk/gdktypes.h>

#include "geometry.h"
#include "polyshape.h"

#include "object.h"
#include "element.h"
#include "connectionpoint.h"
#include "widgets.h"
#include "dia_image.h"

#include "plug-ins.h"

#include "menace.h"
#include "properties.h"

//#include "entites.h"
//#include "classification.h"

///////////////// debut dclaration "define" de paramettre comun  SISSI //////////////

#define ORDINATEUR_LINEWIDTH  0.1
#define ORDINATEUR_FONTHEIGHT 0.8
#define LEFT_SPACE 0.7
#define RIGHT_SPACE 0.3
#define DEFAULT_FONT 0.7
///////////////// fin dclaration "define" de paramettre comun  SISSI //////////////

typedef struct _ObjetSISSI ObjetSISSI;
typedef struct _SISSIDialog SISSIDialog;
typedef struct _SISSIState SISSIState;
typedef struct _SISSIChange SISSIChange;


struct _SISSIState {
  /* menace: */
  // Propriete_Menace proprietes_menaces[42];

  /* autres: */
//   GList *proprietes_autres;
};

struct _SISSIChange {
  ObjectChange obj_change;
  
  ObjetSISSI *obj;
  SISSIState *saved_state;
  int applied;
};


typedef enum {
  ANCHOR_MIDDLE,
  ANCHOR_START,
  ANCHOR_END
} AnchorShape;

static struct _ImageProperties {
  gchar *file;
  gboolean draw_border;
  gboolean keep_aspect;
} default_properties = { "", FALSE, TRUE };


struct _ObjetSISSI {
  Element element;
  ConnectionPoint connections[8];
	// lments de l'image 
  //PolyShape poly;		/* always 1st! */
  AnchorShape 		celltype;
  real 			radius;			/* pseudo-radius */
  int 			subscribers;		/* number of subscribers in this cell,*/
  gboolean 		show_background;
  Color 		fill_colour;
  Text 			*text;
  Color 		line_colour;
  LineStyle 		line_style;
  real 			dashlength;
  real 			line_width;

  DiaImage 		image;
  gchar 		*file;
  real 			border_width;
  Color 		border_color;
    
  gboolean 		draw_border;
  gboolean		keep_aspect;
 
  gchar 		*id_db;   // cet identifiant est alimenter par une base de donne externe est sert uniquement  celle-ci	
	//proprits de classification et d'entit
  gchar 		*confidentialite,*integrite,*disponibilite_echelle;
  float 		disponibilite_valeur;
  gchar 		*name;
  gchar			*entite;
  gchar			*entite_type;
  gchar			*type_element;  /////////// intitul de l'objet///////////////
  //////////////// nom de la salle et du site o se rouve l'objet ////////////////
  gchar			*room,*site;
  
    //////////////menace:///////////////
  GList 		*proprietes_menaces;

  /////////////autres proprietes: ////////////////////////
  GList 		*proprietes_autres;
  /////////////Liste des docs: //////////////////////// 
  GList 		*url_docs; 
  ///////////boite de dialogue
  SISSIDialog 		*properties_dialog;

};


struct _SISSIDialog {
  GtkWidget 		*dialog;
  GtkWidget		*confidentialite, *integrite, *disponibilite_echelle, *disponibilite_valeur;
  GtkWidget		*entite;
  GtkWidget		*name;
 
    //////////////menace:///////////////
  GList 		*proprietes_menaces;
  /////////////autres proprietes: ////////////////////////
  GList 		*proprietes_autres;
  /////////////liste des docs: ////////////////////////
  GList 		*url_docs;
};

typedef struct _Url_Docs Url_Docs;
typedef struct _Url_Docs_Widget Url_Docs_Widget;

struct _Url_Docs {
  gchar *label, *url, *description;
};

struct _Url_Docs_Widget {
  GtkWidget 	*label,*url,*description;
};

typedef struct _Propriete Propriete;
typedef struct _Propriete_Widget Propriete_Widget;

struct _Propriete {
  gchar *label;
  gchar *value;
  gchar *description;
};

struct _Propriete_Widget {
  GtkWidget 	*label,*value,*description;
};


extern Propriete *create_new_propriete_autre(gchar *label, gchar *description, gchar *value);
extern Url_Docs *create_new_url(gchar *label, gchar *description, gchar *value);

extern real object_sissi_distance_from(ObjetSISSI *object_sissi, Point *point);
extern void object_sissi_select(ObjetSISSI *object_sissi, Point *clicked_point, DiaRenderer *interactive_renderer);
extern ObjectChange* object_sissi_move_handle(ObjetSISSI *object_sissi, Handle *handle, Point *to, ConnectionPoint *cp,
			    HandleMoveReason reason, ModifierKeys modifiers);
extern ObjectChange* object_sissi_move(ObjetSISSI *object_sissi, Point *to);
extern void object_sissi_draw(ObjetSISSI *object_sissi, DiaRenderer *renderer);
extern void object_sissi_update_data(ObjetSISSI *pc, AnchorShape horix, AnchorShape vert);

extern void object_sissi_destroy(ObjetSISSI *object_sissi);
void dialog_sissi_destroy(SISSIDialog *properties_dialog);
//extern DiaMenu *pc_get_object_menu(ObjetSISSI *pc, Point *clickedpoint);
extern ObjetSISSI *object_sissi_load(ObjectNode obj_node, int version, const char *filename, ObjetSISSI *object_sissi,Element *elem,DiaObject *obj);

extern GtkWidget *object_sissi_get_properties(ObjetSISSI *object_sissi, gboolean is_default);
extern void object_sissi_apply_properties(ObjetSISSI *object_sissi);

extern DiaObject *object_sissi_copy_using_properties(ObjetSISSI *object_sissi_origine);

extern void object_sissi_save(ObjetSISSI *object_sissi, ObjectNode obj_node, const char *filename);
extern void propriete_menace_write(AttributeNode attr_node, Propriete_Menace *proprietes_menaces);
Propriete_Menace *propriete_menace_read(DataNode composite);
extern void propriete_autre_write(AttributeNode attr_node, Propriete *proprietes_autre);
Propriete *propriete_autre_read(DataNode composite);
extern void url_doc_write(AttributeNode attr_node, Url_Docs *url_docs);
extern Url_Docs *url_doc_read(DataNode composite);
extern GList *vidage_liste_propriete (GList *list);
extern GList *vidage_liste_url_doc (GList *list);

static Propriete propriete_classification_data[] = {
  { N_("Non Protege"), "NON_PROTEGE" ,NULL},
  { N_("Diffusion Restreinte"), "DIFFUSION_RESTREINTE" ,NULL},
  { N_("Confidentiel"), "CONFIDENTIEL" ,NULL},
  { N_("Confidentiel Personnel"), "CONFIDENTIEL_PERSONNEL" ,NULL},
  { N_("Confidentiel Medical"), "CONFIDENTIEL_MEDICAL" ,NULL},
  { N_("Confidentiel Industrie"), "CONFIDENTIEL_INDUSTRIE" ,NULL},
  { N_("Secret"), "SECRET" ,NULL},
  { N_("Secret OTAN"), "SECRET_OTAN" ,NULL},
  { N_("Tres Secret"), "TRES_SECRET" ,NULL},
  { N_("Tres Secret OTAN"), "TRESEC_OTAN" ,NULL},
  { NULL,0,NULL}
};

static Propriete propriete_integrite_data[] = {
  { N_("No integrity"), "NULL" ,NULL},
  { N_("Low integrity"), "FAIBLE" ,NULL},
  { N_("Average sowtware integrity"), "INTEGRITE_LOGICIEL_MOYENNE" ,NULL},
  { N_("Hight sowtware integrity"), "INTEGRITE_LOGICIEL_FORTE" ,NULL},
  { N_("Average hardware integrity"), "INTEGRITE_MATERIEL_MOYENNE" ,NULL},
  { N_("Hight hardware integrity"), "INTEGRITE_MATERIEL_FORTE" ,NULL},
  { NULL,0,NULL}
};

static Propriete propriete_disponibilite_echelle_data[] = {
  { N_("Millie seconde"), "MS" ,NULL},
  { N_("Seconde"), "SECONDE" ,NULL},
  { N_("Minute"), "MINUTE" ,NULL},
  { N_("Hour"), "HEURE" ,NULL},
  { N_("Day"), "JOURNEE" ,NULL},
  { N_("Week"), "SEMAINE" ,NULL},
  { NULL,0,NULL}
};

static Propriete propriete_systeme_data[] = {
  { N_("SYSTEME"), "SYS",NULL},
  { N_("Dispositif d'Acces Internet"), "SYS_INT",NULL},
  { N_("Messagerie"), "SYS_MES",NULL},
  { N_("Intranet"), "SYS_ITR",NULL},
  { N_("Annuaire d'Entreprise"), "SYS_ANU",NULL},
  { NULL,0,NULL}
};

static Propriete propriete_organisation_data[] = {
  { N_("ORGANISATION"), "ORG",NULL},
     { N_("Organisation dont depend L'organisme"), "ORG_DEP",NULL},
     { N_("Organisation de L'organisme"), "ORG_GEN",NULL},
     { N_("Organisation de Projet ou d'un systeme"), "ORG_PRO",NULL},
     { N_("Sous-Traitants/Fournisseurs/Industriels"),"ORG_EXT",NULL},
  { NULL,0,NULL}
};

static Propriete propriete_physique_data[] = {
  { N_("SITE"), "PHY",NULL},
   { N_("Lieu"), "PHY_LIE",NULL},
     { N_("Externe"), "PHY_LIE1",NULL},
     { N_("Locaux"), "PHY_LIE2",NULL},
     { N_("Zone"), "PHY_LIE3",NULL},
   { N_("Service Essentiel"), "PHY_SRV",NULL},
     { N_("Communication"), "PHY_SRV1",NULL},
     { N_("Energie"), "PHY_SRV2",NULL},
     { N_("Refroidissement/Pollution"), "PHY_SRV3",NULL},
  { NULL, 0,NULL}
};

static Propriete propriete_personnel_data[] = {
  { N_("PERSONNEL"), "PER",NULL},
   { N_("Decisionnel"), "PER_DEC",NULL},
   { N_("Utilisateurs"), "PER_UTI",NULL},
   { N_("Exploitant-Maintenance"), "PER_EXP",NULL},
   { N_("Developpeur"), "PER_DEV",NULL},
  { NULL, 0,NULL}
};

static Propriete propriete_reseau_data[] = {
  { N_("RESEAU"), "RES",NULL},
   { N_("Medium et Supports"), "RES_INF",NULL},
   { N_("Relais Passif ou Actif"), "RES_REL",NULL},
   { N_("Interface de Communication"), "RES_INT",NULL},
  { NULL, 0,NULL}
};

static Propriete propriete_logiciel_data[] = {
  { N_("LOGICIEL"), "LOG",NULL},
    { N_("Systeme d'Exploitation"), "LOG_OS",NULL},
    { N_("Logiciel de Service-Maintenance ou administration"), "LOG_SRV",NULL},
    { N_("Progiciel ou Logiciel Standard"), "LOG_STD",NULL},
    { N_("Application Metier"), "LOG_APP",NULL},
      { N_("Application metier standard"), "LOG_APP1",NULL},
      { N_("Application metier specifique"), "LOG_APP2",NULL},
  { NULL, 0,NULL}
};

static Propriete propriete_materiel_data[] = {
  { N_("MATERIEL"), "MAT",NULL},
  { N_("Support de Traitement de Donnees (actif)"), "MAT_ACT",NULL},
  { N_("Materiel transportable"), "MAT_ACT1",NULL},
  { N_("Materiel fixe"), "MAT_ACT2",NULL},
  { N_("Peripherique de traitement"), "MAT_ACT3",NULL},
  { N_("Support electronique"), "MAT_PAS1",NULL},
  { N_("Autres supports"), "MAT_PAS2",NULL},  
  { N_("Support de Donnees (passif)"), "MAT_PAS",NULL},
  { NULL, 0,NULL}
};


static char* liste_image[] = {
  {"shapes/Cisco/workstation.png"},//0
  {"shapes/Cisco/relational_database.png"},//1
  {"shapes/Network/computer.png"},//2 
  {"sheets/SISSI/laptop.png"},//3 ordinateur portable
  {"sheets/SISSI/pda.png"},//4 pda  {"shapes/Cisco/router.png"},//4 routeur
  {"shapes/Cisco/storage_array.png"},//5  completer
  {""},//6  completer
  {""},//7  completer
  {""},//8  completer
  {""},//9  completer
  {"shapes/Cisco/router.png"},//10 routeur
  {"shapes/Cisco/cloud.png"},//11 nuage rseau
  {"shapes/Cisco/phone.png"},//12 Telephone
  {"shapes/Cisco/cellular_phone.png"},//13 Telephone
  {"shapes/Cisco/hub.png"},//14  hub
  {"shapes/Cisco/modem.png"},//15  modeme
  {"shapes/Cisco/key.png"},//16  boitier de chiffrement
  {""},//17  completer
  {""},//18  completer
  {""},//19  completer
  {""},//20  completer
  {"shapes/Cisco/diskette.png"},//21 support magnetique
  {"shapes/Misc/file.png"},//22 support papier
  {""},//23 Telephone
  {""},//24 Telephone
  {""},//25  hub
  {""},//26  modeme
  {""},//27  completer
  {""},//28  completer
  {""},//29  completer
  {""},//30  completer
  {""},//31  completer
  {"shapes/Cisco/file_cabinet.png"},//32 armoire
  {"sheets/SISSI/safe.png"},//33 coffre
  {""},//34 Telephone
  {""},//35  hub
  {""},//36  modeme
  {""},//37  completer
  {""},//38  completer
  {"sheets/SISSI/personne.xpm"},//39
  {"sheets/SISSI/si.xpm"},//40 SI tiers
  {"sheets/SISSI/analyse.xpm"},//41 analyse de log
  {"sheets/SISSI/sauvegarde.xpm"},//42 politique de sauvegarde
  {"sheets/SISSI/info.xpm"},//43  
  {"sheets/SISSI/logiciel.xpm"},//44  
  {""},//45  
  {""},//46  completer
  {""},//47
  {""},//48 
  {""},//49  completer
  {"sheets/SISSI/barriere.png"},//50  
  {"sheets/SISSI/camera_interphone.png"},//51  
  {"sheets/SISSI/capteur_brise_glace.png"},//52  completer
  {"sheets/SISSI/capteur_faisceau.png"},//53  completer
  {"sheets/SISSI/condensateur.png"},//54  completer
  {"sheets/SISSI/detecteur_alimentation.png"},//55  completer
  {"sheets/SISSI/detecteur_courant.png"},//56  
  {"sheets/SISSI/detecteur_enthalpie.png"},//57  completer
  {"sheets/SISSI/detecteur_fumee.png"},//58  completer
  {"sheets/SISSI/detecteur_humidite.png"},//59  completer
  {"sheets/SISSI/detecteur_lumiere.png"},//60  completer
  {"sheets/SISSI/detecteur_mouvement.png"},//61  
  {"sheets/SISSI/detecteur_mouvement_video.png"},//62  completer
  {"sheets/SISSI/detecteur_niveau.png"},//63  completer
  {"sheets/SISSI/detecteur_pression.png"},//64  completer
  {"sheets/SISSI/detecteur_qualite_air.png"},//65  completer
  {"sheets/SISSI/detecteur_tension.png"},//66  
  {"sheets/SISSI/detecteur_vehicule.png"},//67  
  {"sheets/SISSI/detecteur_vibration.png"},//68  completer
  {"sheets/SISSI/digi_code.png"},//69  completer
  {"sheets/SISSI/extracteur_humidite.png"},//70  completer
  {"sheets/SISSI/filtre_air.png"},//71  completer
  {"sheets/SISSI/lecteur_badge_clavier.png"},//72  completer
  {"sheets/SISSI/lecteur_badge.png"},//73  completer
  {"sheets/SISSI/lecteur_biometrique.png"},//74  completer
  {"sheets/SISSI/pompe.png"},//75  completer
  {"sheets/SISSI/pompe_rotative.png"},//76  
  {"sheets/SISSI/porte_tambour.png"},//77  
  {"sheets/SISSI/ventilateur_axial.png"},//78  completer
  {"sheets/SISSI/ventilateur_centrifuge.png"},//79  completer
  {"sheets/SISSI/ventilateur.png"},//80  completer
  {"sheets/SISSI/onduleur.png"},//81  completer
  {"sheets/SISSI/extincteur.png"},//81  completer
};

static char* liste_element[] = {
  {"poste bureautique"},//0
  {"SGBD"},//1
  {"ordinateur"},//2 
  {"ordinateur portable"},//3 ordinateur portable
  {"PDA"},//4 pda  {"shapes/Cisco/router.png"},//4 routeur
  {"Baie de stockage"},//5  completer
  {""},//6  completer
  {""},//7  completer
  {""},//8  completer
  {""},//9  completer
  {"Routeur"},//10 routeur
  {"Nuage reseau"},//11 nuage rseau
  {"Telephone"},//12 Telephone
  {"Telephone portable"},//13 Telephone
  {"HUB"},//14  hub
  {"Modem"},//15  modeme
  {"Boitier de chiffrement"},//16  boitier de chiffrement
  {""},//17  completer
  {""},//18  completer
  {""},//19  completer
  {""},//20  completer
  {"Bande magnetique"},//21 support magnetique
  {"Support papier"},//22 support papier
  {""},//23 Telephone
  {""},//24 Telephone
  {""},//25  hub
  {""},//26  modeme
  {""},//27  completer
  {""},//28  completer
  {""},//29  completer
  {""},//30  completer
  {""},//31  completer
  {"Armoire"},//32 armoire
  {"Coffre"},//33 Telephone
  {""},//34 Telephone
  {""},//35  hub
  {""},//36  modeme
  {""},//37  completer
  {""},//38  completer
  {"Personne"},//39
  {"SI tiers"},//40 SI tiers
  {"Analyse de log"},//41 analyse de log
  {"Plan de sauvegarde"},//42 politique de sauvegarde
  {"Information"},//43  
  {"Logiciel"},//44  
  {""},//45  
  {""},//46  completer
  {""},//47
  {""},//48 
  {""},//49  completer
  {"arriere"},//50  
  {"camera interphone"},//51  
  {"capteur brise glace"},//52  completer
  {"capteur faisceau"},//53  completer
  {"condensateur"},//54  completer
  {"detecteur alimentation"},//55  completer
  {"detecteur courant"},//56  
  {"detecteur enthalpie"},//57  completer
  {"detecteur fumee"},//58  completer
  {"detecteur humidite"},//59  completer
  {"detecteur lumiere"},//60  completer
  {"detecteur mouvement"},//61  
  {"detecteur mouvement video"},//62  completer
  {"detecteur niveau"},//63  completer
  {"detecteur pression"},//64  completer
  {"detecteur qualite air"},//65  completer
  {"detecteur tension"},//66  
  {"detecteur vehicule"},//67  
  {"detecteur vibration"},//68  completer
  {"digi code"},//69  completer
  {"extracteur humidite"},//70  completer
  {"filtre air"},//71  completer
  {"lecteur badge clavier"},//72  completer
  {"lecteur badge"},//73  completer
  {"lecteur biometrique"},//74  completer
  {"pompe"},//75  completer
  {"pompe rotative"},//76  
  {"porte tambour"},//77  
  {"ventilateur axial"},//78  completer
  {"ventilateur centrifuge"},//79  completer
  {"ventilateur"},//80  completer
  {"onduleur"},//81  completer
  {"extincteur"},//82  completer
};


/////////////////dfinition des type de couleur  /////////////////
static Color color_green = { 0.0f, 1.0f, 0.0f };
static Color color_green_clear = { 0.87, 0.98f, 0.91 };
static Color color_red = { 1.0f, 0.0f, 0.0f };
static Color color_blue = { 0.0f, 0.0f, 1.0f };
static Color color_brown = { 0.54f, 0.08f, 0.08f };
static Color color_pink = { 1.0f, 0.75f, 0.80f };
static Color color_yellow = { 1.0f, 1.0f, 0.0f };
static Color color_orange = { 0.2f, 0.5f, 0.5f };
static Color color_orange_clear = { 0.1f, 1.0f, 0.5f };
static Color color_gris = { 0.5f, 0.5f, 0.5f };
static Color color_gris_clear = { 0.95f, 0.95f, 0.95f };

#endif /* SISSI_H */
